"""Pull a YellowAnt application's data into a file"""
import json, yaml
import click

from yacli.cli import pass_context
from yacli.constants import APP_FILENAME, DEVELOPER_APPLICATION_ENDPOINT
from yacli.common import save_app_config, read_app_id_from_config, save_application_yaml, AppIdInvalid, AppDataCorrupt,\
FilePermissionError





@click.command()
@click.argument('app')
@click.option("--filename", default=APP_FILENAME, type=click.Path())
@pass_context
def cli(ctx, app, filename):
    """Pull application data from host in JSON format. Either provide application id or invoke name as argument."""
    try:
        ctx.is_auth_valid()

        try:
            app = int(app)
        except:
            app = read_app_id_from_config(app)

        if app is None:
            raise AppIdInvalid("Please check if application invoke name details are present in the file: app.yellowant")

        req = ctx.get(DEVELOPER_APPLICATION_ENDPOINT + str(app) + "/")
        if req.status_code == 200:
            try:
                application_json = json.loads(req.content)
            except:
                raise AppDataCorrupt("Application data could not be parsed.")

            # client_id = application_json.get("client_id", None)
            # client_secret = application_json.get("client_secret", None)
            # if client_id is None or client_secret is None:
            #     raise AppDataCorrupt("Application data is incomplete.")
            # ctx.log("\nCLIENT ID: " + client_id)
            # ctx.log("CLIENT SECRET: " + client_secret)
            
            try:
                save_application_yaml(application_json)
            except:
                raise FilePermissionError("Could not save application data a local file. Please check permissions.")
            
            ctx.log("Successfully saved application data to {}".format(filename))

            save_app_config(application_json["id"], application_json["invoke_name"])
        else:
            print(req.text)
            raise Exception("Either your auth credentials are incorrect, " +
                            "or the application could not be found.")
    except Exception as e:
        ctx.log(e)



import json, yaml

try: # python 2 imports
    import ConfigParser
except: # python 3 imports
    import configparser as ConfigParser

from yacli.constants import APP_CONFIG_FILENAME, APP_FILENAME


class AppIdInvalid(Exception):
    pass

class AppDataCorrupt(Exception):
    pass

class FilePermissionError(Exception):
    pass


def save_app_config(app_id, app_invoke_name, config_file=APP_CONFIG_FILENAME):
    """Write token and host configs to a file"""
    config = ConfigParser.RawConfigParser()
    config.read(config_file)
    if not config.has_section(app_invoke_name):
        config.add_section(app_invoke_name)
    config.set(app_invoke_name, 'invoke_name', app_invoke_name)
    config.set(app_invoke_name, 'id', app_id)
    with open(config_file, 'w') as configfile:
        config.write(configfile)

def read_app_id_from_config(app_invoke_name, config_file=APP_CONFIG_FILENAME):
    config = ConfigParser.ConfigParser()
    config.read(config_file)
    
    if not config.has_section(app_invoke_name) or not config.has_option(app_invoke_name, "id"):
        return None

    return config.get(app_invoke_name, "id")

def save_application_json(application_json, app_filename=APP_FILENAME):
    application_json.pop("client_id")
    application_json.pop("client_secret")
    with open(app_filename, "w") as outfile:
        json.dump(application_json, outfile, indent=4, sort_keys=True)

def save_application_yaml(application_json, app_filename=APP_FILENAME):
    with open(app_filename, "w") as outfile:
        yaml.safe_dump(application_json, outfile, allow_unicode=True, default_flow_style=False)



from twython import TwythonStreamer, TwythonStreamError

from .config import (
    app_key, app_secret, oauth_token, oauth_token_secret, unittest
)


class TwythonStreamTestCase(unittest.TestCase):
    def setUp(self):
        class MyStreamer(TwythonStreamer):
            def on_success(self, data):
                self.disconnect()

            def on_error(self, status_code, data):
                raise TwythonStreamError(data)

        self.api = MyStreamer(app_key, app_secret,
                              oauth_token, oauth_token_secret)

        client_args = {
            'headers': {
                'User-Agent': '__twython__ Stream Test'
            }
        }
        # Initialize with header for coverage checking for User-Agent
        self.api_with_header = MyStreamer(app_key, app_secret,
                                          oauth_token, oauth_token_secret,
                                          client_args=client_args)

    @unittest.skip('skipping non-updated test')
    def test_stream_status_filter(self):
        self.api.statuses.filter(track='twitter')

    @unittest.skip('skipping non-updated test')
    def test_stream_status_sample(self):
        self.api.statuses.sample()

    @unittest.skip('skipping non-updated test')
    def test_stream_status_firehose(self):
        self.assertRaises(TwythonStreamError, self.api.statuses.firehose,
                          track='twitter')

    @unittest.skip('skipping non-updated test')
    def test_stream_site(self):
        self.assertRaises(TwythonStreamError, self.api.site,
                          follow='twitter')

    @unittest.skip('skipping non-updated test')
    def test_stream_user(self):
        self.api.user(track='twitter')



"""Create a YellowAnt application with a developer account"""
import json, yaml
import click

from yacli.cli import pass_context
from yacli.constants import APP_FILENAME, DEVELOPER_APPLICATION_ENDPOINT
from yacli.common import read_app_id_from_config, save_app_config, save_application_yaml, \
AppIdInvalid, AppDataCorrupt, FilePermissionError


@click.command()
@click.argument('app')
@click.option("--filename", default=APP_FILENAME, type=click.File("rb"))
@pass_context
def cli(ctx, app, filename):
    """Update an application with your developer account."""
    try:
        ctx.is_auth_valid()

        try:
            app = int(app)
        except:
            app = read_app_id_from_config(app)
        
        if app is None:
            raise AppIdInvalid("Please check if application invoke name details are present in the file: app.yellowant")

        data = yaml.safe_load(filename.read())
        req = ctx.put(DEVELOPER_APPLICATION_ENDPOINT + str(app) + "/", data=json.dumps(data))
        if req.status_code == 200:
            try:
                application_json = json.loads(req.content)
            except:
                raise AppDataCorrupt("Application data could not be parsed.")

            # client_id = application_json.get("client_id", None)
            # client_secret = application_json.get("client_secret", None)
            # if client_id is None or client_secret is None:
            #     raise AppDataCorrupt("Application data is incomplete.")
            # ctx.log("\nCLIENT ID: " + client_id)
            # ctx.log("CLIENT SECRET: " + client_secret)

            try:
                save_application_yaml(application_json)
                save_app_config(application_json["id"], application_json["invoke_name"])
            except:
                raise FilePermissionError("Could not update application data a local file. Please check permissions.")

            ctx.log("Successfully updated application {} with your YellowAnt developer account."
                    .format(application_json["invoke_name"]))
        elif req.status_code == 400:
            error = json.loads(req.text)
            msg = "\nError: Please check your app data JSON.\n"
            print(error)
            for key_name in error:
                msg += key_name + ": " + str(error[key_name][0]) + "\n"
            raise Exception(msg)
        else:
            print(req.text)
            error = json.loads(req.text)
            raise Exception(error["detail"])
    except Exception as e:
        ctx.log(e)




#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages

__author__ = 'Vishwa Krishnakumar <vishwa@yellowant.com>'
__version__ = '0.0.22'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

setup(
    name='yellowant',
    version=__version__,
    install_requires=['requests>=2.1.0', 'requests_oauthlib>=0.4.0', 'click', 'socketIO-client==0.7.2'],
    author='Vishwa Krishnakumar',
    author_email='vishwa@yellowant.com',
    license=open('LICENSE').read(),
    url='https://github.com/vishwa306/yellowant-python-sdk/tree/master',
    keywords='yellowant slack bot',
    description='Python wrapper for the YellowAnt API',
    zip_safe=True,
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Communications :: Chat',
        'Topic :: Internet'
    ],
    entry_points={
        "console_scripts": [
            "yellowant=yacli.cli:cli"
        ],
    }
)
